//**************************************************************************//
//* FRIDA: fast reliable interactive data analysis                         *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://frida.sourceforge.net                                           *//
//* yamc.cpp                                                               *//
//* read YAML data files, using libyaml                                    *//
//**************************************************************************//

#include <string>
using namespace std;
#include "coord.h"
#include "yamc.h"
#include "mystd.h"

CYamlRead::CYamlRead( FILE *F_in )
{
    memset( &parser, 0, sizeof(parser) );
    memset( &event, 0, sizeof(event) );
    if (!yaml_parser_initialize( &parser ))
        throw string("Could not initialize the parser object");
    yaml_parser_set_input_file( &parser, F_in );
    last_valid = "START";
}

CYamlRead::~CYamlRead()
{
    yaml_parser_delete( &parser );
}

void CYamlRead::next()
{
    if( !yaml_parser_parse( &parser, &event) )
        throw string( "Invalid input? Last valid entry was \"" ) + 
            last_valid + "\"";
}

void CYamlRead::done()
{
    yaml_event_delete( &event);
}

void CYamlRead::checktype( int type, string errmsg )
{
    next();
    if( event.type != type )
        throw errmsg;
    if( event.type == YAML_SCALAR_EVENT )
        last_valid = (const char*)event.data.scalar.value;
    done();
}

void CYamlRead::checkvalue( string value )
{
    next();
    if( event.type != YAML_SCALAR_EVENT )
        throw string("No scalar while expecting ") + value;
    if( strcmp((const char*)event.data.scalar.value, value.c_str()) )
        throw string("Found ") + (const char*)event.data.scalar.value +
            " instead of " + value;
    last_valid = (const char*)event.data.scalar.value;
    done();
}

void CYamlRead::skipvalues( int repet )
{
    for( int i=0; i<repet; ++i ){
        checktype( YAML_SCALAR_EVENT, string( "skip " ) +  strg(repet) );
    }
}

bool CYamlRead::scalar_from_seq( string seqname, string *ret )
{
    next();
    if( event.type == YAML_SEQUENCE_END_EVENT ){
        done();
        return false;
    }
    if( event.type != YAML_SCALAR_EVENT )
        throw string( "No scalar in sequence " ) + seqname +
            "; last valid entry was \"" + last_valid + "\"";
    *ret = (const char*)event.data.scalar.value;
    last_valid = (const char*)event.data.scalar.value;
    done();
    return true;
}

bool CYamlRead::pair_from_map( string mapname, string *key, string *val )
{
    next();
    if( event.type == YAML_MAPPING_END_EVENT ){
        done();
        return false;
    }
    if( event.type != YAML_SCALAR_EVENT )
        throw string( "No scalar key in mapping " ) + mapname +
            "; last valid entry was \"" + last_valid + "\"";
    *key = (const char*)event.data.scalar.value;
    done();
    next();
    if( event.type != YAML_SCALAR_EVENT )
        throw string( "No scalar value in mapping " ) + mapname +
            "; last valid entry was \"" + last_valid + "\"";
    *val = (const char*)event.data.scalar.value;
    last_valid = (const char*)event.data.scalar.value;
    done();
    return true;
}

string CYamlRead::getscalar( string scalarname )
{
    string ret;
    next();
    if( event.type != YAML_SCALAR_EVENT )
        throw string( "No scalar " ) + scalarname +
            "; last valid entry was \"" + last_valid + "\"";
    ret = (const char*)event.data.scalar.value;
    done();
    return ret;
}

bool CYamlRead::getCoord( string label, CCoord *co )
{
    next();
    if( event.type == YAML_MAPPING_END_EVENT )
        return false;
    if( event.type != YAML_SCALAR_EVENT )
        throw string( "inconsistency where " + label + " expected" );
    if( strcmp((const char*)event.data.scalar.value, label.c_str()) )
        throw string("Found ") + (const char*)event.data.scalar.value +
            " instead of " + label;
    last_valid = (const char*)event.data.scalar.value;
    done();
    checktype( YAML_MAPPING_START_EVENT, label + ": no map\n" );
    checkvalue( "name" );
    co->name = getscalar( label + ".name" );
    checkvalue( "unit" );
    co->unit = getscalar( label + ".unit" );
    checktype( YAML_MAPPING_END_EVENT, label + ": no end of map\n" );
    return true;
}

